//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Instrument/BackgroundItems.h
//! @brief     Defines BackgroundItem classes
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_INSTRUMENT_BACKGROUNDITEMS_H
#define BORNAGAIN_GUI_MODEL_INSTRUMENT_BACKGROUNDITEMS_H

#include "GUI/Model/Session/SessionItem.h"

class IBackground;

class BA_CORE_API_ BackgroundItem : public SessionItem {
public:
    explicit BackgroundItem(const QString& model_type);

    virtual std::unique_ptr<IBackground> createBackground() const = 0;
};

class NoBackgroundItem : public BackgroundItem {
public:
    static constexpr auto M_TYPE{"NoBackground"};

    NoBackgroundItem();
    std::unique_ptr<IBackground> createBackground() const override;
};

class ConstantBackgroundItem : public BackgroundItem {
private:
    static constexpr auto P_VALUE{"BackgroundValue"};

public:
    static constexpr auto M_TYPE{"ConstantBackground"};

    ConstantBackgroundItem();
    std::unique_ptr<IBackground> createBackground() const override;

    double backgroundValue() const;
    void setBackgroundValue(double value);
};

class PoissonNoiseBackgroundItem : public BackgroundItem {
public:
    static constexpr auto M_TYPE{"PoissonNoiseBackground"};

    PoissonNoiseBackgroundItem();
    std::unique_ptr<IBackground> createBackground() const override;
};

#endif // BORNAGAIN_GUI_MODEL_INSTRUMENT_BACKGROUNDITEMS_H
