//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/MultiLayerItem.h
//! @brief     Defines class MultiLayerItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_MULTILAYERITEM_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_MULTILAYERITEM_H

#include <heinz/Vectors3D.h>
#include "GUI/Model/Session/SessionItem.h"

class LayerItem;
class ItemWithMaterial;
class DoubleDescriptor;
class VectorDescriptor;

class BA_CORE_API_ MultiLayerItem : public SessionItem {
private:
    static constexpr auto P_CROSS_CORR_LENGTH{"CrossCorrelationLength"};
    static constexpr auto P_EXTERNAL_FIELD{"ExternalField"};
    static constexpr auto T_LAYERS{"Layer tag"};
    static constexpr auto P_DESCRIPTION{"Description"};

public:
    static constexpr auto M_TYPE{"MultiLayer"};

    MultiLayerItem();

    QVector<ItemWithMaterial*> itemsWithMaterial() const;

    QString description() const;
    void setDescription(const QString& description);

    DoubleDescriptor crossCorrLength() const;
    static bool isCrossCorrLengthPropertyName(const QString& name);

    R3 externalField() const;
    void setExternalField(const R3& ext_field);
    SessionItem* externalFieldItem() const;
    VectorDescriptor externalFieldVector() const;

    QVector<LayerItem*> layers() const;
    SessionItem* layersItem() const;

    //! Creates and inserts a layer at given index.
    //! No properties etc. have been initialized, this  has to be done by the caller
    //! If index = -1, create a layer at the end ot the list.
    LayerItem* addLayer(int index);

    void removeLayer(LayerItem* item);
    void moveLayer(LayerItem* item, LayerItem* beforeThisLayer);

private:
    void updateLayers();
};

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_MULTILAYERITEM_H
