//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Sample/ParticleLayoutItem.h
//! @brief     Defines class ParticleLayoutItem
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SAMPLE_PARTICLELAYOUTITEM_H
#define BORNAGAIN_GUI_MODEL_SAMPLE_PARTICLELAYOUTITEM_H

#include "GUI/Model/Group/SelectionDescriptor.h"
#include "GUI/Model/Session/SessionModel.h"

class InterferenceItem;
class ItemWithParticles;
class DoubleDescriptor;

class BA_CORE_API_ ParticleLayoutItem : public SessionItem {
private:
    static constexpr auto P_TOTAL_DENSITY{"TotalParticleDensity"};
    static constexpr auto P_WEIGHT{"Weight"};
    static constexpr auto T_PARTICLES{"Particle Tag"};
    static constexpr auto T_INTERFERENCE{"Interference Tag"};

public:
    static constexpr auto M_TYPE{"ParticleLayout"};

    ParticleLayoutItem();

    DoubleDescriptor totalDensity() const;
    DoubleDescriptor weight() const;

    QVector<ItemWithParticles*> particles() const;
    void addParticle(ItemWithParticles* particle);
    void removeParticle(ItemWithParticles* particle);

    SelectionDescriptor<InterferenceItem*> interference() const;
    template <typename T> T* createInterference();
    void setInterference(InterferenceItem* interference);
    void updateDensityValue();

private:
    void updateDensityAppearance();
};

template <typename T> T* ParticleLayoutItem::createInterference()
{
    static_assert(std::is_base_of<InterferenceItem, T>::value,
                  "Class must be derived from InterferenceFunciontItem");

    return model()->insertItem<T>(this, -1, T_INTERFERENCE);
}

#endif // BORNAGAIN_GUI_MODEL_SAMPLE_PARTICLELAYOUTITEM_H
