//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Model/Session/SessionItemUtils.h
//! @brief     Defines namespace GUI::Session::ItemUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_MODEL_SESSION_SESSIONITEMUTILS_H
#define BORNAGAIN_GUI_MODEL_SESSION_SESSIONITEMUTILS_H

#include <heinz/Vectors3D.h>
#include <QString>
#include <QVariant>

class SessionItem;
class GroupInfo;

namespace GUI::Session::ItemUtils {

//! Returns the index of the given item within its parent. Returns -1 when no parent is set.
int ParentRow(const SessionItem& item);

//! Returns type of variant (additionally checks for user type).
int VariantType(const QVariant& variant);

//! Returns true if variants has compatible types.
bool CompatibleVariantTypes(const QVariant& oldValue, const QVariant& newValue);

//! Returns true if given variants have same type and value.
//! For custom variants (e.g. ComboProperty) will always return false (see
//! explanations in cpp file).

bool IsTheSame(const QVariant& var1, const QVariant& var2);

} // namespace GUI::Session::ItemUtils

#endif // BORNAGAIN_GUI_MODEL_SESSION_SESSIONITEMUTILS_H
