//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Instrument/EditDistributionDialog.cpp
//! @brief     Implements class EditDistributionDialog
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Instrument/EditDistributionDialog.h"
#include "GUI/Model/Instrument/ItemWithDistribution.h"
#include "GUI/View/Instrument/DistributionEditor.h"
#include "GUI/View/Instrument/DistributionPlot.h"

#include <QDialogButtonBox>
#include <QSplitter>
#include <QVBoxLayout>


EditDistributionDialog::EditDistributionDialog(ItemWithDistribution* item, const QString& title,
                                               const std::optional<MeanConfig>& mean_config,
                                               GUI::ID::Distributions distributions,
                                               QWidget* parent, Qt::WindowFlags f)
    : QDialog(parent, f), m_item(item)
{
    setMinimumSize(256, 256);
    resize(800, 480);
    setWindowTitle(title);

    auto* layout = new QVBoxLayout(this);

    auto* splitter = new QSplitter(this);
    layout->addWidget(splitter);

    m_plot = new DistributionPlot(this);
    m_plot->setItem(m_item->distribution());
    splitter->addWidget(m_plot);

    m_selector = new DistributionSelector(mean_config, distributions, this);
    m_selector->setItem(m_item);
    splitter->addWidget(m_selector);

    splitter->setSizes({500, 300});

    auto* bbox = new QDialogButtonBox(this);
    bbox->addButton(QDialogButtonBox::Ok);
    bbox->addButton(QDialogButtonBox::Cancel);
    connect(bbox, &QDialogButtonBox::accepted, this, &QDialog::accept);
    connect(bbox, &QDialogButtonBox::rejected, this, &QDialog::close);
    layout->addWidget(bbox);

    setSizeGripEnabled(true);

    connect(m_selector, &DistributionSelector::distributionChanged, this,
            &EditDistributionDialog::updatePlot);
}

void EditDistributionDialog::updatePlot()
{
    // no need to update the selector because that's the only widget which changes the item
    m_plot->setItem(m_item->distribution());
}
