//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Resample/Flux/ScalarFlux.h
//! @brief     Defines class ScalarFlux.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifdef SWIG
#error no need to expose this header to Swig
#endif

#ifndef USER_API
#ifndef BORNAGAIN_RESAMPLE_FLUX_SCALARFLUX_H
#define BORNAGAIN_RESAMPLE_FLUX_SCALARFLUX_H

#include <heinz/Complex.h>
#include "Resample/Flux/IFlux.h"

//! Specular reflection and transmission coefficients in a layer in case
//! of scalar interactions between the layers and the scattered particle.

class ScalarFlux : public IFlux {
public:
    ScalarFlux(complex_t kz, Eigen::Vector2cd TR);

    // The following functions return the transmitted and reflected amplitudes
    // for different incoming beam polarizations and eigenmodes
    Eigen::Vector2cd T1plus() const override;
    Eigen::Vector2cd R1plus() const override;
    Eigen::Vector2cd T2plus() const override;
    Eigen::Vector2cd R2plus() const override;
    Eigen::Vector2cd T1min() const override;
    Eigen::Vector2cd R1min() const override;
    Eigen::Vector2cd T2min() const override;
    Eigen::Vector2cd R2min() const override;

    //! Returns z-part of the two wavevector eigenmodes
    Eigen::Vector2cd getKz() const override;

    complex_t getScalarT() const;
    complex_t getScalarR() const;
    complex_t getScalarKz() const { return m_kz; }

private:
    const complex_t m_kz; //!< Signed vertical wavevector component k_z

    //! Transmission and reflection coefficient
    //!
    //! In the manual called A^{-} = m_TR(0) and A^{+} = m_TR(1).
    //! Values of the transmitted/reflected (=down/up propagating) wavefunction
    //! at top boundary of the layer (resp. at the bottom of the top air/vacuum layer).
    const Eigen::Vector2cd m_TR;

    const Eigen::Vector2cd m_plus;
    const Eigen::Vector2cd m_min;
};

#endif // BORNAGAIN_RESAMPLE_FLUX_SCALARFLUX_H
#endif // USER_API
