//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Sample/Particle/ParticleCoreShell.h
//! @brief     Defines ParticleCoreShell.
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_SAMPLE_PARTICLE_PARTICLECORESHELL_H
#define BORNAGAIN_SAMPLE_PARTICLE_PARTICLECORESHELL_H

#include "Sample/Particle/IParticle.h"

class Particle;

//! A particle with a core/shell geometry.
//! @ingroup samples

class ParticleCoreShell : public IParticle {
public:
    inline static const std::string class_name = "ParticleCoreShell";
    std::string className() const final { return class_name; }

    ParticleCoreShell(const Particle& shell, const Particle& core,
                      R3 relative_core_position = R3(0.0, 0.0, 0.0));
    ~ParticleCoreShell() override;

    ParticleCoreShell* clone() const override;

    void accept(INodeVisitor* visitor) const override { visitor->visit(this); }

    SlicedParticle createSlicedParticle(const ZLimits& limits) const override;

    const Particle* coreParticle() const;

    const Particle* shellParticle() const;

    std::vector<const INode*> getChildren() const override;

protected:
    void addAndRegisterCore(const Particle& core, R3 relative_core_position);
    void addAndRegisterShell(const Particle& shell);
    ParticleCoreShell();

    std::unique_ptr<Particle> m_shell;
    std::unique_ptr<Particle> m_core;
};

inline const Particle* ParticleCoreShell::coreParticle() const
{
    return m_core.get();
}

inline const Particle* ParticleCoreShell::shellParticle() const
{
    return m_shell.get();
}

#endif // BORNAGAIN_SAMPLE_PARTICLE_PARTICLECORESHELL_H
