#include "Base/Element/DepthProbeElement.h"
#include "Base/Axis/FixedBinAxis.h"
#include "Base/Vector/Direction.h"
#include "Tests/GTestWrapper/google_test.h"

class DepthProbeElementTest : public ::testing::Test {
protected:
    DepthProbeElementTest();
    DepthProbeElement createDefaultElement();

    std::unique_ptr<FixedBinAxis> m_z_positions;
};

DepthProbeElementTest::DepthProbeElementTest() : m_z_positions(new FixedBinAxis("z", 10, 0.0, 10.0))
{
}

DepthProbeElement DepthProbeElementTest::createDefaultElement()
{
    const double wavelength = 1.0;
    const double angle = 2.0;
    return DepthProbeElement(wavelength, angle, m_z_positions.get());
}

TEST_F(DepthProbeElementTest, InitialState)
{
    const double wavelength = 1.0;
    const double angle = 2.0;
    const double phi = 0.0;
    const R3 k_i = vecOfLambdaAlphaPhi(wavelength, angle, phi);
    const DepthProbeElement& ele = createDefaultElement();

    EXPECT_EQ(wavelength, ele.wavelength());
    EXPECT_EQ(angle, ele.getAlphaI());
    bool intensity_comparison_result = (ele.getIntensities() == 0.0).min();
    EXPECT_TRUE(intensity_comparison_result);
    EXPECT_EQ(k_i, ele.getKi());
    EXPECT_TRUE(ele.isCalculated());
}
