// ************************************************************************** //
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      Wrap/Swig/libBornAgainSample.i
//! @brief     SWIG interface file for libBornAgainSample
//!
//!            Configuration is done in Sample/CMakeLists.txt
//!
//! @homepage  http://apps.jcns.fz-juelich.de/BornAgain
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2013
//! @authors   Scientific Computing Group at MLZ Garching
//! @authors   C. Durniak, G. Pospelov, W. Van Herck, J. Wuttke
//
// ************************************************************************** //

%module(directors="1", moduleimport="import $module") "libBornAgainSample"

%include "commons.i"

%include "../../auto/Wrap/doxygenSample.i"

%include "ignoreBase.i"
%include "ignoreSample.i"

// force swig to use move ctor instead of copy ctor
%typemap(out) SlicedParticle %{
    $result = SWIG_NewPointerObj(new $1_ltype(std::move($1)), $&1_descriptor, SWIG_POINTER_OWN);
  %}

%feature("director") ISampleNode;            // needed by IFormFactor
%feature("director") IFormFactor;        // needed by IBornFF
%feature("director") IBornFF;    // used in CustomFormFactor.py

%{
#include "Base/Vector/Transform3D.h"
#include "Param/Distrib/ParameterDistribution.h"
#include "Sample/Aggregate/Interference1DLattice.h"
#include "Sample/Aggregate/Interference2DLattice.h"
#include "Sample/Aggregate/Interference2DParaCrystal.h"
#include "Sample/Aggregate/Interference2DSuperLattice.h"
#include "Sample/Aggregate/Interference3DLattice.h"
#include "Sample/Aggregate/InterferenceFinite2DLattice.h"
#include "Sample/Aggregate/InterferenceFinite3DLattice.h"
#include "Sample/Aggregate/InterferenceHardDisk.h"
#include "Sample/Aggregate/InterferenceNone.h"
#include "Sample/Aggregate/InterferenceRadialParaCrystal.h"
#include "Sample/Aggregate/InterferenceTwin.h"
#include "Sample/Aggregate/ParticleLayout.h"
#include "Sample/Correlations/IPeakShape.h"
#include "Sample/HardParticle/FormFactorAnisoPyramid.h"
#include "Sample/HardParticle/FormFactorBar.h"
#include "Sample/HardParticle/FormFactorBox.h"
#include "Sample/HardParticle/FormFactorCantellatedCube.h"
#include "Sample/HardParticle/FormFactorCone.h"
#include "Sample/HardParticle/FormFactorCone6.h"
#include "Sample/HardParticle/FormFactorCosineRipple.h"
#include "Sample/HardParticle/FormFactorCuboctahedron.h"
#include "Sample/HardParticle/FormFactorCylinder.h"
#include "Sample/HardParticle/FormFactorDodecahedron.h"
#include "Sample/HardParticle/FormFactorEllipsoidalCylinder.h"
#include "Sample/HardParticle/FormFactorFullSphere.h"
#include "Sample/HardParticle/FormFactorFullSpheroid.h"
#include "Sample/HardParticle/FormFactorHemiEllipsoid.h"
#include "Sample/HardParticle/FormFactorHollowSphere.h"
#include "Sample/HardParticle/FormFactorIcosahedron.h"
#include "Sample/HardParticle/FormFactorLongBoxGauss.h"
#include "Sample/HardParticle/FormFactorLongBoxLorentz.h"
#include "Sample/HardParticle/FormFactorPrism3.h"
#include "Sample/HardParticle/FormFactorPrism6.h"
#include "Sample/HardParticle/FormFactorPyramid.h"
#include "Sample/HardParticle/FormFactorSawtoothRipple.h"
#include "Sample/HardParticle/FormFactorTetrahedron.h"
#include "Sample/HardParticle/FormFactorTruncatedCube.h"
#include "Sample/HardParticle/FormFactorTruncatedSphere.h"
#include "Sample/HardParticle/FormFactorTruncatedSpheroid.h"
#include "Sample/Lattice/BakeLattice.h"
#include "Sample/Lattice/ISelectionRule.h"
#include "Sample/Material/MaterialFactoryFuncs.h"
#include "Sample/Multilayer/Layer.h"
#include "Sample/Multilayer/MultiLayer.h"
#include "Sample/Particle/Crystal.h"
#include "Sample/Particle/FormFactorCrystal.h"
#include "Sample/Particle/FormFactorWeighted.h"
#include "Sample/Particle/MesoCrystal.h"
#include "Sample/Particle/Particle.h"
#include "Sample/Particle/ParticleComposition.h"
#include "Sample/Particle/ParticleCoreShell.h"
#include "Sample/Particle/SlicedParticle.h"
#include "Sample/Scattering/IFormFactorDecorator.h"
#include "Sample/Interface/LayerInterface.h"
#include "Sample/Interface/LayerRoughness.h"
#include "Sample/SoftParticle/FormFactorGauss.h"
#include "Sample/SoftParticle/FormFactorSphereGaussianRadius.h"
#include "Sample/SoftParticle/FormFactorSphereLogNormalRadius.h"
%}

%include "fromBase.i"
%include "fromParam.i"

%newobject Interference2DLattice::createSquare(double lattice_length, double xi);
%newobject Interference2DLattice::createHexagonal(double lattice_length, double xi);
%newobject Interference2DParaCrystal::createSquare(
        double lattice_length, double damping_length, double domain_size_1, double domain_size_2);
%newobject Interference2DParaCrystal::createHexagonal(
        double lattice_length, double damping_length, double domain_size_1, double domain_size_2);
%newobject Interference2DSuperLattice::createSquare(
        double lattice_length, double xi, unsigned size_1, unsigned size_2);
%newobject Interference2DSuperLattice::createHexagonal(
        double lattice_length, double xi, unsigned size_1, unsigned size_2);
%newobject InterferenceFinite2DLattice::createSquare(
        double lattice_length, double xi, unsigned size_1, unsigned size_2);
%newobject InterferenceFinite2DLattice::createHexagonal(
        double lattice_length, double xi, unsigned size_1, unsigned size_2);

%template(swig_dummy_type_inode_vector) std::vector<INode*>;
%template(swig_dummy_type_const_inode_vector) std::vector<const INode*>;

%template(vector_IFormFactorPtr_t) std::vector<IFormFactor*>;

%include "Sample/Material/Material.h"
%include "Sample/Material/MaterialFactoryFuncs.h"

%include "Sample/Scattering/ISampleNode.h"
%include "Sample/Scattering/IFormFactor.h"
%include "Sample/Scattering/IBornFF.h"
%include "Sample/Scattering/IFormFactorDecorator.h"
%include "Sample/Scattering/Rotations.h"

%include "Sample/Particle/FormFactorCrystal.h"
%include "Sample/Particle/FormFactorWeighted.h"
%include "Sample/Particle/IAbstractParticle.h"
%include "Sample/Particle/Crystal.h"
%include "Sample/Particle/IParticle.h"
%include "Sample/Particle/MesoCrystal.h"
%include "Sample/Particle/Particle.h"
%include "Sample/Particle/ParticleComposition.h"
%include "Sample/Particle/ParticleCoreShell.h"

%include "Sample/Correlations/FTDecay1D.h"
%include "Sample/Correlations/FTDecay2D.h"
%include "Sample/Correlations/FTDistributions1D.h"
%include "Sample/Correlations/FTDistributions2D.h"
%include "Sample/Correlations/FTDecay1D.h"
%include "Sample/Correlations/FTDecay2D.h"
%include "Sample/Correlations/IPeakShape.h"

%include "Sample/Aggregate/IInterference.h"
%include "Sample/Aggregate/Interference1DLattice.h"
%include "Sample/Aggregate/Interference2DLattice.h"
%include "Sample/Aggregate/Interference2DParaCrystal.h"
%include "Sample/Aggregate/Interference2DSuperLattice.h"
%include "Sample/Aggregate/Interference3DLattice.h"
%include "Sample/Aggregate/InterferenceFinite2DLattice.h"
%include "Sample/Aggregate/InterferenceFinite3DLattice.h"
%include "Sample/Aggregate/InterferenceHardDisk.h"
%include "Sample/Aggregate/InterferenceNone.h"
%include "Sample/Aggregate/InterferenceRadialParaCrystal.h"
%include "Sample/Aggregate/InterferenceTwin.h"
%include "Sample/Aggregate/ParticleLayout.h"

%include "Sample/Interface/LayerRoughness.h"

%include "Sample/Multilayer/Layer.h"
%include "Sample/Multilayer/MultiLayer.h"

// SWIG workaround for using axes units the same way as they are used in cpp files
%rename(RoughnessModel) RoughnessModelWrap;
%include "Sample/Multilayer/RoughnessModels.h"

%include "Sample/HardParticle/IFormFactorPolyhedron.h"
%include "Sample/HardParticle/IFormFactorPrism.h"
%include "Sample/HardParticle/IProfileRipple.h"

%include "Sample/HardParticle/FormFactorAnisoPyramid.h"
%include "Sample/HardParticle/FormFactorBox.h"
%include "Sample/HardParticle/FormFactorCantellatedCube.h"
%include "Sample/HardParticle/FormFactorCone.h"
%include "Sample/HardParticle/FormFactorCone6.h"
%include "Sample/HardParticle/FormFactorCosineRipple.h"
%include "Sample/HardParticle/FormFactorCuboctahedron.h"
%include "Sample/HardParticle/FormFactorCylinder.h"
%include "Sample/HardParticle/FormFactorDodecahedron.h"
%include "Sample/HardParticle/FormFactorEllipsoidalCylinder.h"
%include "Sample/HardParticle/FormFactorFullSphere.h"
%include "Sample/HardParticle/FormFactorFullSpheroid.h"
%include "Sample/HardParticle/FormFactorHemiEllipsoid.h"
%include "Sample/HardParticle/FormFactorHollowSphere.h"
%include "Sample/HardParticle/FormFactorIcosahedron.h"
%include "Sample/HardParticle/FormFactorLongBoxGauss.h"
%include "Sample/HardParticle/FormFactorLongBoxLorentz.h"
%include "Sample/HardParticle/FormFactorPrism3.h"
%include "Sample/HardParticle/FormFactorPrism6.h"
%include "Sample/HardParticle/FormFactorPyramid.h"
%include "Sample/HardParticle/FormFactorSawtoothRipple.h"
%include "Sample/HardParticle/FormFactorTetrahedron.h"
%include "Sample/HardParticle/FormFactorTruncatedCube.h"
%include "Sample/HardParticle/FormFactorTruncatedSphere.h"
%include "Sample/HardParticle/FormFactorTruncatedSpheroid.h"

%include "Sample/SoftParticle/FormFactorGauss.h"
%include "Sample/SoftParticle/FormFactorSphereGaussianRadius.h"
%include "Sample/SoftParticle/FormFactorSphereLogNormalRadius.h"

%include "Sample/Lattice/ISelectionRule.h"
%include "Sample/Lattice/Lattice3D.h"
%include "Sample/Lattice/Lattice2D.h"
%include "Sample/Lattice/BakeLattice.h"

%extend Vec3<double> {
    Vec3<double> __add__(const Vec3<double>& rhs) const {
        return *($self) + rhs; }
    Vec3<double> __mul__(double c) const {
        return c * *($self); }
    Vec3<double> __rmul__(double c) const {
        return *($self) * c; }
    Vec3<double> __neg__() const {
        return - *($self); }
};
